<?php
/**
 * Warning: require PHP5
 * Xoops Object CRUD handler
 * 
 * @copyright      RB Lin (xtheme@gmail.com>
 * @license        WTFPL Version 2 - http://sam.zoy.org/wtfpl/
 * @author         RB Lin (xtheme@gmail.com>
 * @url            http://code.google.com/p/xoops-rb-framework/
 * @version        2.1
 */
 
class RbCrud extends XoopsObject
{
	var $className;
	
	function __construct($className = '')
	{
		$this->className = $className;
	}
	
	function RbCrud($className = '')
	{
		$this->__construct($className);
	}
	
	function __set($key, $val)
	{
		$field = strtolower($key);
		if (!empty($key) && isset($val) && isset($this->vars[$field])) {
			return $this->setVar($field, $val);
		}
    }

	function __get($key)
	{
		$field = strtolower($key);
		//if (null != $this->getVar($field)) {
		if (isset($this->vars[$field])) {
			return $this->getVar($field);
		}
		//trigger_error(__FILE__ .' on line '. __LINE__  .'<br />Class '.$this->className.' extends '. __CLASS__ .'::'. __FUNCTION__ .' does not define field '.$field, E_USER_ERROR);
	}
	
	function __call($method, $args) {
		// For security, first word must be "set" or "get"
		// Upcase real call to method name first word after "set" or "get"
		if (preg_match_all('/(set|get)([A-Z][a-z0-9]+)+/i', $method, $words)) {
			$firstWord  = $words[1][0]; // set or get
			$methodName = ucfirst(array_shift($words[2]));
			//$field = strtolower($methodName);
			//if (null != $this->getVar($field)) return $this->getVar($field);
			if (method_exists($this, $methodName)) {
				$methodObj = array($this, $methodName);
				if ($firstWord == 'set') {
					return call_user_func_array($methodObj, $args);
				} else {
					return call_user_func_array($methodObj, null);
				}
			}
		}
		
		// Magic getVar
		$field = strtolower($method);
		if (isset($this->vars[$field])) {
			$methodObj = array($this, 'getVar');
			$args = array_merge((array)$field, $args);
			return call_user_func_array($methodObj, $args);
		}
		trigger_error(__FILE__ .' on line '. __LINE__  .'<br />Class '.$this->className.' extends '. __CLASS__ .'::'. $method .'() not found', E_USER_ERROR);
	}
	
}

class RbCrudHandler extends XoopsObjectHandler
{
	var $tableName;     // Table name, 資料表名稱
	var $keyName;       // Primary key  auto_increment field name, 主欄位
	var $className;     // Class name
	var $orderBy;       // Defaule ORDER BY, 預設排序方式
	var $errors         = array();
	
	// Active Records
	
	var $ar_distinct    = '';
	var $ar_select      = array();
	var $ar_join		= array();
	var $ar_where       = array();
	var $ar_groupby     = array();
	var $ar_having      = array();
	var $ar_orderby     = array();
	var $ar_limit       = false;
	var $ar_offset      = false;
	var $ar_data        = array();
	
	var $setToken       = false;
	var $getToken       = false;
	var $debug          = false;
	
	
	function __construct($db = null, $tableName = '', $className = '', $keyName = '', $orderBy = '')
	{
		$db = ( is_object($db) && is_a($db, 'XoopsDatabase') ) ? $db : $GLOBALS['xoopsDB'];
		$table = $db->prefix($tableName);
		$this->XoopsObjectHandler($db);
		
		$this->tableName = $table;
		$this->keyName   = $keyName;
		$this->className = $className;
		$this->orderBy   = $orderBy;
	}
	
	
	function RbCrudHandler($db = null, $tableName = '', $className = '', $keyName = '', $orderBy = '')
	{
		$this->__construct($db, $tableName, $className, $keyName, $orderBy);
	}
	
	// --------------------------------------------------------------------
	
	/**
	* Object Action Function
	* ORM (Object-relational mapping) 
	* use Object do CRUD
	* 物件操控方法, UD 前必須先建立物件
	*
	* Create    $handler->create() 
	* Read      $handler->get(), $handler->getObjects()
	* Update    $handler->insert()
	* Delete    $handler->delete()
	*/
	
	/**
	* Create Object
	* 建立 (宣告) 新物件
	*
	* @param	bool
	* @return	object
	*/
	function &create($isNew = true)
	{
		$object =& new $this->className();
		if ($isNew === true) {
			$object->setNew();
		}
		return $object;
	}
	
	
	/**
	* Get a Object by Primary key or other field
	* Only return first data
	* 藉由主鍵或其他欄位取得一筆物件化的資料
	* 若符合的資料超過一筆, 也只回傳第一筆物件
	*
	* @access	public
	* @param	string
	* @param	string	field name
	* @return	object
	*/
	function get($value = '', $field = '')
	{	
		$object = false;
		
		if (!empty($value)) {
			if (empty($field)) {
				$field = $this->keyName;
			}
			$clause = "`{$field}` = ".$this->db->quoteString($value);
			$this->where($clause);
		} 
		
		if (!count($this->ar_where)) {
			trigger_error(__CLASS__ .'::'. __FUNCTION__ .' need where condition', E_USER_WARNING);
			return $object;
		}
		
		$sql = $this->compileSQL();
		
		if (!$result = $this->db->query($sql)) {
            return $object;
        }
		
		if ($this->db->getRowsNum($result) != 1) {
            return $object;
        }
		
		$object =& $this->create(false);
        $object->assignVars($this->db->fetchArray($result));
		
		return $object;
	}
	
	
	/**
	* Action
	* Return array of object or array data by search
	* 依照查詢條件回傳符合資料
	*
	* @param	bool	return object or array
	* @param	bool	use the Primary as key for the array
	* @return	array
	*/
	function &getAll($setWhere = null, $setOrderBy = null, $setLimit = null, $setOffset = null, $asObject = true, $id_as_key = true)
	{
		/*
		* Compatibility ver.1 start
		*/
		if ($setWhere || $setOrderBy || $setLimit || $setOffset) {
			
			$this->resetSQL();
			
			if ($setWhere) {
				$this->where($setWhere);
			}
			
			if ($setOrderBy) {
				$this->order_by($setOrderBy);
			}
			
			if ($setLimit) {
				$this->limit($setLimit);
			}
			
			if ($setOffset) {
				$this->offset($setOffset);
			}
		}
		/*
		* Compatibility ver.1 end
		*/
		
		// Add Default ORDER BY
		if ((count($this->ar_orderby) == 0) && !empty($this->orderBy)) {
			$dob = explode(' ', $this->orderBy);
			$this->order_by($dob[0], $dob[1]);
		}
		
		$sql = $this->compileSQL();
		//echo $sql.'<br /><br />';
		$ret = array();
		
		if (!$result = $this->db->query($sql)) {
			return $ret;
		}
		
		if ($asObject) {
			while ($myrow = $this->db->fetchArray($result)) {
				$object =& $this->create(false);
				$object->assignVars($myrow);
				if ( $id_as_key ) {
					$ret[$myrow[$this->keyName]] = $object;
				} else {
					$ret[] = $object;
				}
				unset($object);
			}
		} else {
			while ($myrow = $this->db->fetchArray($result)) {
				if ($id_as_key) {
					$ret[$myrow[$this->keyName]] = $myrow;
				} else {
					$ret[] = $myrow;
				}
			}
		}
		return $ret;
	}
	
	
	/**
	* Action
	* Return object data by search
	* 將查詢結果的每筆資料都物件化並以陣列回傳
	*
	* @param	bool	use the Primary as key for the array
	* @return	array of objects
	*/
	function getObjects($setWhere = null, $setOrderBy = null, $setLimit = null, $setOffset = null, $id_as_key = false)
	{
		$ret = $this->getAll($setWhere, $setOrderBy, $setLimit, $setOffset, true, false);
		return $ret;
	}
	
	
	/**
	* Action
	* use Primary as key, some field as value, return array
	* 指定一個欄位, 回傳一組鍵為主鍵, 值為指定欄位的陣列
	*
	* @param	string	field name
	* @return	array
	*/
	function getList($identifier = '', $id_as_key = true)
	{
		$ret = array();
		
		$this->ar_select[] = $this->keyName;
		$this->ar_select[] = $identifier;
		$sql = $this->compileSQL();
		
		if (!$result = $this->db->query($sql)) {
			return $ret;
		}
		
		$myts =& MyTextSanitizer::getInstance();
		while ($myrow = $this->db->fetchArray($result)) {
			if ($id_as_key) {
				$ret[$myrow[$this->keyName]] = $myts->htmlSpecialChars($myrow[$identifier]);
			} else {
				$ret[] = $myts->htmlSpecialChars($myrow[$identifier]);
			}
		}
		return $ret;
	}
	
	
	/**
	* Action
	* return Primary array by search
	* 將查詢結過回傳只有主鍵的陣列
	*
	* @return	array
	*/
	function getIds()
	{
		$ret = array();
		
		$this->select($this->keyName);
		$sql = $this->compileSQL();
		
		if (!$result = $this->db->query($sql)) {
			return $ret;
		}
		
		while ($myrow = $this->db->fetchArray($result)) {
			$ret[] = $myrow[$this->keyName];
		}
		return $ret;
	}
	
	
	/**
	* Insert or Update a Object data
	* 新增或更新一筆物件資料
	*
	* @param	object
	* @param	bool         test or not
	* @return	bool
	*/
	function insert(&$object, $force = true)
	{
		if (!is_a($object, $this->className)) {
			return false;
		}
		
		if (!$object->isDirty()) {
			return true;
		}
		
		if (!$object->cleanVars()) {
			return false;
		}
		
		$queryFunc = empty($force) ? 'query' : 'queryF';
		
		$insert_into = array();
		$insert_set  = array();
		$update_set  = array();
		foreach ( $object->cleanVars as $key => $val ) {
			${$key} = $val;
			$insert_into[] = sprintf('`%s`', $key);
			$insert_set[]  = $this->db->quoteString($val);
			$update_set[]  = sprintf('`%s`', $key).' = '.$this->db->quoteString($val);
		}
		$insert_into = implode(', ', $insert_into);
		$insert_set  = implode(', ', $insert_set);
		$update_set  = implode(', ', $update_set);
		
		if ($object->isNew()) {
			$sql  = 'INSERT INTO '.$this->tableName;
			$sql .= ' ('.$insert_into.') VALUES ('.$insert_set.')';

		} else {
			$sql  = 'UPDATE '.$this->tableName.' SET '.$update_set;
			$sql .= ' WHERE `'.$this->keyName.'` = '.$this->db->quoteString($object->getVar($this->keyName));
		}
		
		if (!$result = $this->db->{$queryFunc}($sql)) {
			$this->setErrors(__CLASS__ .'::'. __FUNCTION__ .' query fail');
			return false;
		}
		
		if ( !$object->getVar($this->keyName) && $object_id = $this->db->getInsertId() ) {
			$object->assignVar($this->keyName, $object_id);
		}
		return true;
	}
	
	
	/**
	* Delete a Object data
	* 刪除一筆物件資料
	*
	* @param	object
	* @param	bool         test or not
	* @return	bool
	*/
	function delete(&$object, $force = true)
	{
		if (!is_a($object, $this->className)) {
			$this->setErrors(__CLASS__ .'::'. __FUNCTION__ .' is not an object');
			return false;
		}
		$sql = sprintf('DELETE FROM %s WHERE '.$this->keyName.' = %u', $this->tableName, $object->getVar($this->keyName));
		
		$queryFunc = empty($force) ? 'query' : 'queryF';
		
		if (!$result = $this->db->{$queryFunc}($sql)) {
			$this->setErrors(__CLASS__ .'::'. __FUNCTION__ .' query fail');
			return false;
		}
		return true;
	}
	
	
	// --------------------------------------------------------------------
	
	/**
	* Row Action Function
	* use Primary key do CRUD
	* 以主鍵作對單筆資料作更快速的操作
	*/
	
	
	/**
	* Insert a row data
	* 寫入一筆資料
	*
	* @param	array
	* @param	bool
	* @param	bool
	* @return	bool	link $this->insert
	*
	* $handler->insertRow($_POST)
	*/
	function insertRow($data, $force = true, $not_gpc = false)
	{
		if ($this->setToken AND !$this->getToken) {
			return false;
		}
		
		if (count($this->ar_data) > 0) {
			$data = array_merge($data, $this->ar_data);
		}
		
		$object =& $this->create(true);
		$object->setVars($data, $not_gpc);
		return $this->insert($object, $force);
	}
	
	
	/**
	* Update a row data by Primary Key
	* 藉由主鍵更新一筆資料
	*
	* @param	array		request data
	* @param	integer	Primary Key, if request data without Primary Key
	* @param	bool
	* @param	bool
	* @return	bool		link $this->insert
	*
	* $handler->updateRow($_POST)
	*/
	function updateRow($data, $primary = 0, $force = true, $not_gpc = false)
	{
		
		if ($this->setToken AND !$this->getToken) {
			return false;
		}
		
		$primary = intval($primary);
		
		if (!$primary) {
			if (array_key_exists($this->keyName, $data)) {
				$primary = $data[$this->keyName];
			} else {
				$this->setErrors(__CLASS__ .'::'. __FUNCTION__ .' need primary key');
				return false;
			}
		}
		
		if (count($this->ar_data) > 0) {
			$data = array_merge($data, $this->ar_data);
		}
		
		$object =& $this->get($primary);
		$object->setVars($data, $not_gpc);
		return $this->insert($object, $force);
	}
	
	
	/**
	* Delete a row data by Primary Key
	* 藉由主鍵刪除一筆資料
	*
	* @param	integer	Primary Key
	* @return	bool
	*
	* $handler->deleteRow($_POST['id'])
	*/
	function deleteRow($primary = 0, $force = true)
	{
		if ($this->setToken AND !$this->getToken) {
			return false;
		}
		
		$primary = intval($primary);
		
		$object = $this->get($primary);
		return $this->delete($object, $force);
	}
	
	
	// --------------------------------------------------------------------
	
	/**
	* Batch Action Function
	*  批次處理的方法
	*/
	
	/**
	* Delete data after Query set
	* 刪除符合條件的資料
	*
	* @param	bool
	* @param	bool
	* @return	bool
	*
	* $handler->where('status = 0')->order('create_time ASC')->limit('5')->deleteAll()
	*/
	function deleteAll($force = true, $as_object = false)
	{
		if ($this->setToken AND !$this->getToken) {
			return false;
		}
		
		if ($as_object) {
			$objects = $this->getObjects();
			$num = 0;
			foreach (array_keys($objects) as $key) {
				$num += $this->delete($objects[$key], $force) ? 1 : 0;
			}
			unset($objects);
			return $num;
		}
		
		$queryFunc = empty($force) ? 'query' : 'queryF';
		
		if (!count($this->ar_where)) {
			trigger_error(__CLASS__ .'::'.__FUNCTION__ .' need where condition', E_USER_WARNING);
			return false;
		}
		
		$sql_override = 'DELETE FROM '.$this->tableName;
		$sql = $this->compileSQL($sql_override);
		//echo $sql.'<br /><br />';
		if (!$this->db->{$queryFunc}($sql)) {
			$this->setErrors(__CLASS__ .'::'. __FUNCTION__ .' query error<br />'.$sql.'<br />');
			return false;
		}
		return true;
	}
	
	
	/**
	* Update date after Query set
	* 更新符合條件的資料欄位
	*
	* @param 	array
	* @param	bool
	* @return	bool
	*
	* $handler->where('status = 1')->updateAll($_POST)
	*/
	function updateAll($data, $force = true)
	{
		if ($this->setToken AND !$this->getToken) {
			return false;
		}
		
		$queryFunc = empty($force) ? 'query' : 'queryF';
		
		if (is_array($data) && count($data) > 0) {
			$set_arr = array();
			foreach ($data as $key => $val) {
				$update_set = "`{$key}` = ";
				if (is_array($val)) {
					$update_set .= $this->db->quoteString(implode(',', $val));
				} else {
					$update_set .= $this->db->quoteString($val);
				}
				$set_arr[] = $update_set;
			}
			$set_clause = implode(', ', $set_arr);
		} else {
			//$this->setErrors(__CLASS__ .'::'. __FUNCTION__ .' no data input!!<br />');
			//return false;
			$set_clause = $data;
		}
		
		$sql_override = 'UPDATE `'.$this->tableName.'` SET ' . $set_clause;
		$sql = $this->compileSQL($sql_override);
		
		if (!$this->db->{$queryFunc}($sql)) {
			$this->setErrors(__CLASS__ .'::'. __FUNCTION__ .' query error<br />'.$sql.'<br />');
			return false;
		}
		return true;
    }
	
	
	// --------------------------------------------------------------------
	
	/**
	* Count Action Function
	* 查詢數量的方法
	*/
	
	/**
	* Count query result and return int
	* 依照查詢條件回傳符合的加總數
	*
	* $handler->where('cat = 1 AND type = 2')->getCounts()
	*/
	function getCount($setWhere = null)
	{
		return $this->getCounts($setWhere);
	}
	
	function getCounts($setWhere = null)
	{
		/*
		* Compatibility ver.1 start
		*/
		if ($setWhere) {
			$this->resetSQL();
			$this->where($setWhere);
		}
		/*
		* Compatibility ver.1 end
		*/
		
		$this->select('COUNT(*)');
		$sql = $this->compileSQL();
		//echo $sql.'<br /><br />';
		if (!$result = $this->db->query($sql)) {
			return 0;
		}
		list($count) = $this->db->fetchRow($result);
		return $count;
	}
	
	
	/**
	* SELECT COUNT(DISTINCT column_name) FROM table_name
	*
	* $handler->where('cat = 1')->getCountDistinct('field')
	*/
	function getCountDistinct($field = '')
	{
		$this->ar_distinct = '';    // force unset
		$this->ar_select = array(); // force unset
		
		$sql = sprintf('COUNT(DISTINCT %s)', $field);
		$this->select($sql);
		
		$sql = $this->compileSQL();
		//echo $sql.'<br /><br />';
		if (!$result = $this->db->query($sql)) {
			return 0;
		}
		list($count) = $this->db->fetchRow($result);
		return $count;
	}
	
	
	/**
	* Get number of rows in result and return int
	* 依照查詢條件回傳符合的資料筆數
	*
	* $handler->where('cat = 1 AND type = 2')->getRowsNum()
	*/
	function getRowsNum($setWhere = null)
	{
		/*
		* Compatibility ver.1 start
		*/
		if ($setWhere) {
			$this->resetSQL();
			$this->where($setWhere);
		}
		/*
		* Compatibility ver.1 end
		*/
		
		$this->select('COUNT(*)');
		$sql = $this->compileSQL();
		//echo $sql.'<br /><br />';
		if (!$result = $this->db->query($sql)) {
			return 0;
		}
		$count = $this->db->getRowsNum($result);
		return $count;
	}
	

	/**
	* Check data is exists and return bool
	* 依照查詢條件回傳布林值
	*
	* @param	string
	* @param	string
	* @return	bool
	*
	* $handler->where('id = 5')->isExists()
	* $handler->isExists(5)
	* $handler->isExists('7654321', 'isbn')
	*/
	/**
	* Deprecated Method
	* 
	* Suggest Use
	* $object = $handler->get(5);
	* or
	* $object = $handler->get('7654321', 'isbn');
	* Then Check $object is Exists
	* if ($object) { .... }
	*/
	function isExists($value= '', $field = '', $setWhere = null)
	{
		/*
		* Compatibility ver.1 start
		*/
		if (is_array($field)) {
			$setWhere = $field;
			$field = '';
		}
		
		if ($setWhere) {
			$this->resetSQL();
			$this->where($setWhere);
		}
		/*
		* Compatibility ver.1 end
		*/
		
		if (!empty($value)) {
			if (empty($field)) {
				$field = $this->keyName;
			}
			$clause = "`{$field}` = ".$this->db->quoteString($value);
			//$this->ar_where[] = $clause;
			$this->where($clause);
		}
		
		if (!count($this->ar_where)) {
			return false;
		}
		
		$this->select('COUNT(*)');
		$sql = $this->compileSQL();
		if ($result = $this->db->query($sql)) {
			list($count) = $this->db->fetchRow($result);
			if ( $count > 0 ){
				return true;
			}
		}
		return false;
	}
	
	// --------------------------------------------------------------------
	
	/**
	*  Field Action Function
	*  針對一個欄位的方法
	*/
	
	/**
	* Get a field value
	* 取得欄位值, 一次只能查詢一個欄位
	*
	* @param	string
	* @param	string	MIN, MAX, SUM, AVG
	* @return	string
	*/
	function getField($field, $type = '')
	{
		$ret = '';
		
		$type = (in_array(strtoupper(trim($type)), array('MIN', 'MAX', 'SUM', 'AVG'), true)) ? $type : '';
		if ($type != '') {
			$selectFunc = 'select_'.strtolower($type);
			$this->{$selectFunc}($field);
		} else {
			$this->select($field);
		}
		
		if (count($this->ar_select) != 1) {
			trigger_error(__CLASS__ .'::'. __FUNCTION__ .' need where condition', E_USER_WARNING);
			return $ret;
		}
		
		$sql = $this->compileSQL();
		$result = $this->db->query($sql);
		list($ret) = $this->db->fetchRow($result);
		return $ret;
	}
	
	
	/**
	* Dec a field value
	* If without $this-where() will update all data
	* 遞增欄位值
	* 請搭配 $this->wher() 設定更新條件, 否則將更新所有資料
	*
	* @param	string
	* @param	integer
	* @return	bool
	*
	* $handler->where('id = 7')->incField('views', 1)
	*/
	function incField($field, $step = 1)
	{
		$step = intval($step);
		$clause = "$field = $field + $step";
		$sql_override = 'UPDATE `'.$this->tableName.'` SET '.$clause;
		$sql = $this->compileSQL($sql_override);
		$result = $this->db->queryF($sql);
		if ($result) {
			return true;
		}
		return false;
	}
	
	/**
	* Dec a field value
	* If without $this-where() will update all data
	* 遞減欄位值
	* 請搭配 $this->wher() 設定更新條件, 否則將更新所有資料
	*
	* @param	string
	* @param	integer
	* @return	bool
	*
	* $handler->where('uid = 7')->decField('karma', 5)
	*/
	function decField($field, $step = 1)
	{
		$step = intval($step);
		$clause = "$field = $field - $step";
		$sql_override = 'UPDATE `'.$this->tableName.'` SET '.$clause;
		$sql = $this->compileSQL($sql_override);
		$result = $this->db->queryF($sql);
		if ($result) {
			return true;
		}
		return false;
	}
	
	
	// --------------------------------------------------------------------
	
	
	/**
	* Set the INSERT data
	* 設定一些用來寫入的資料
	*
	* @param	array
	* @return	this
	*/
	function data($field = '', $value = '')
	{
		if (is_array($field)) {
			$this->data_arr($value);
		} else {
			if ($value != '') {
				$this->ar_data[$field] = $value;
			}
		}
		return $this;
	}
	
	function data_arr($arr)
    {
        foreach ($arr as $key => $val) {
            $this->data($key, $val);
        }
		return $this;
    }
	
	/**
	* Check token
	*
	* $handler->token()->deleteRow($_POST['id'])
	* $handler->token()->where('status = 1')->updateAll($_POST)
	*/
	function token()
	{
		$this->setToken = true;
		$this->getToken = true;
		
		if( substr( XOOPS_VERSION , 6 , 3 ) > 2.0 ) {
			global $GLOBALS;
			// XOOPS 2.1+
			if (!$GLOBALS['xoopsSecurity']->check()) {
				$this->getToken = false;
			}
		} else {
			// XOOPS 2.0.x
			$token_handler = new XoopsMultiTokenHandler();
			if(!$token_handler->autoValidate($token_name)) {
				$this->getToken = false;
			}
		}
		
		return $this;
	}
	
	
	/**
	* SQL Action Function
	* Active Record
	*/
	
	
	/**
	* Generates a SELECT MIN(field) portion of a query
	* 取得指定欄位 "最小" 值
	*
	* @param	string
	* @param	string
	* @return	object
	*/
	function select_min($field, $alias = '')
	{
		if (!is_string($field) OR $field == '') {
			trigger_error(__CLASS__ .'::'. __FUNCTION__ .' need field', E_USER_WARNING);
		}
		
		$alias = ($alias != '') ? $alias : $field;
		
		$str = 'MIN('.trim($field).') AS '.trim($alias);
		$this->select($str);
		return $this;
	}
	
	
	/**
	* Generates a SELECT MAX(field) portion of a query
	* 取得指定欄位 "最大" 值
	*
	* @param	string
	* @param	string
	* @return	object
	*/
	function select_max($field, $alias = '')
	{
		if (!is_string($field) OR $field == '') {
			trigger_error(__CLASS__ .'::'. __FUNCTION__ .' need field', E_USER_WARNING);
		}

		$alias = ($alias != '') ? $alias : $field;
		
		$str = 'MAX('.trim($field).') AS '.trim($alias);
		$this->select($str);
		return $this;
	}
	
	
	/**
	* Generates a SELECT SUM(field) portion of a query
	* 取得指定欄位 "加總" 值
	*
	* @param	string
	* @param	string
	* @return	object
	*/
	function select_sum($field, $alias = '')
	{
		if (!is_string($field) OR $field == '') {
			trigger_error(__CLASS__ .'::'. __FUNCTION__ .' need field', E_USER_WARNING);
		}

		$alias = ($alias != '') ? $alias : $field;
		
		$str = 'SUM('.trim($field).') AS '.trim($alias);
		$this->select($str);
		return $this;
	}
	
	
	/**
	* Generates a SELECT AVG(field) portion of a query
	* 取得指定欄位 "平均" 值
	*
	* @param	string
	* @param	string
	* @return	object
	*/
	function select_avg($field, $alias = '')
	{
		if (!is_string($field) OR $field == '') {
			trigger_error(__CLASS__ .'::'. __FUNCTION__ .' need field', E_USER_WARNING);
		}

		$alias = ($alias != '') ? $alias : $field;
		
		$str = 'AVG('.trim($field).') AS '.trim($alias);
		$this->select($str);
		return $this;
	}
	
	
	/**
	* Set the SELECT condition
	* 設定 SELECT 條件
	*
	* @param	string
	* @return	object
	*/
	function select($value)
	{
		if ($value != '') {
			if (is_array($value)) {
				$this->select_arr($value);
			} else {
				$this->ar_select[] = $value;
			}
		}
		return $this;
	}
	
	function select_arr($arr)
    {
        foreach ($arr as $val) {
            $this->select($val);
        }
		return $this;
    }
	
	/**
	* DISTINCT
	*
	* Set a flag which tells the query string compiler to add DISTINCT
	*
	* @access	public
	* @param	bool
	* @return	object
	*/
	function distinct($field = '')
	{
		$this->ar_distinct = !empty($field) ? $field : '';
		return $this;
	}
	
	
	/**
	* Sets the WHERE value
	* 設定 WHERE 條件
	*
	* @param	string
	* @return	object
	*/
	function where($value)
	{
		if ($value != '') {
			if (is_array($value)) {
				$this->where_arr($value);
			} else {
				$this->ar_where[] = $value;
			}
		}
		return $this;
	}
	
	function where_arr($arr)
    {
        foreach ($arr as $val) {
            $this->where($val);
        }
		return $this;
    }
	
	/**
	* Sets the GROUP BY value
	* 設定 GROUP BY 條件
	*
	* @param	string
	* @return	object
	*/
	function group_by($value)
	{
		if (is_string($value)) {
			$value = explode(',', $value);
		}
		
		foreach ($value as $val) {
			$val = trim($val);
			
			if ($val != '') {
				$this->ar_groupby[] = $val;
			}
		}
		return $this;
	}
	
	function groupby($value)
	{
		return $this->group_by($value);
	}
	
	/**
	* Set the HAVING condition
	* 設定 HAVING 條件
	*
	* @param	string
	* @return	object
	*/
	function having($value)
	{
		if (is_string($value)) {
			$value = explode(',', $value);
		}
		
		foreach ($value as $val) {
			$val = trim($val);
			
			if ($val != '') {
				$this->ar_having[] = $val;
			}
		}
		return $this;
	}
	
	
	/**
	* Set the ORDER BY condition
	* 設定 ORDER BY  條件
	*
	* @access	public
	* @param	string
	* @param 	string
	* @return	object
	*
	* $handler->order_by('created DESC')  =  " ORDER BY created DESC "
	* $handler->order_by('created')  =  " ORDER BY created ASC "
	*/
	function order_by($orderby, $direction = '')
	{
		if (trim($orderby) == '')
		{
			return $this;
		}
		
		if (trim($direction) != '')
		{
			$orderby = sprintf('`%s`', $orderby);
			$direction = (in_array(strtoupper(trim($direction)), array('ASC', 'DESC'), true)) ? ' '.$direction : ' ASC';
		}
		
		$orderby_clause = $orderby.$direction;
		
		$this->ar_orderby[] = $orderby_clause;
		// array_push($this->ar_orderby, $orderby_clause);
		return $this;
	}
	
	function orderby($orderby, $direction = '')
	{
		return $this->order_by($orderby, $direction);
	}
	
	/**
	* Set the LIMIT condition
	*
	* @access	public
	* @param	integer	the limit value
	* @param	integer	the offset value
	* @return	object
	*/
	function limit($value, $offset = '')
	{
		$this->ar_limit = intval($value);
		
		if ($offset != '') {
			$this->ar_offset = intval($offset);
		}
		return $this;
	}
	
	
	/**
	* Set the OFFSET condition
	*
	* @access	public
	* @param	integer	the offset value
	* @return	object
	*/
	function offset($offset)
	{
		$this->ar_offset = intval($offset);
		return $this;
	}
	
	/**
	* Combination query string
	* 組合查詢的 sql 字串
	*
	* @param     string
	* @return	string
	*/
	function compileSQL($sql_override = false, $sql_reset = true)
	{
		$sql = empty($this->ar_distinct) ? 'SELECT ' : 'SELECT DISTINCT '.$this->ar_distinct.' ';
		
		$sql .= (count($this->ar_select) == 0) ? '*' : implode(', ', $this->ar_select);
		
		$sql .= ' FROM '.$this->tableName;
		
		if ($sql_override != false) {
			$sql = $sql_override;
		}
		
		if (count($this->ar_where) > 0) {
			$sql .= "\nWHERE ";
			$sql .= implode("\nAND ", $this->ar_where);
		}
		
		if (count($this->ar_groupby) > 0) {
			$sql .= "\nGROUP BY ";
			$sql .= implode(', ', $this->ar_groupby);
		}
		
		if (count($this->ar_having) > 0) {
			$sql .= "\nHAVING ";
			$sql .= implode(', ', $this->ar_having);
		}
		
		if (count($this->ar_orderby) > 0) {
			$sql .= "\nORDER BY ";
			$sql .= implode(', ', $this->ar_orderby);
		}
		
		if (is_numeric($this->ar_limit) AND !empty($this->ar_limit)) {
			if (is_numeric($this->ar_offset)) {
				$sql .= "\nLIMIT ".$this->ar_offset.", ".$this->ar_limit;
			} else {
				$sql .= "\nLIMIT ".$this->ar_limit;
			}
		}
		
		if ($sql_reset != false) {
			$this->resetSQL();
		}
		
		if ($this->debug == true) {
			$this->setErrors($sql);
			$this->debug = false;
			echo $this->getHtmlErrors();
		}
		
		return $sql;
	}
	
	
	/**
	* Render SQL
	* 輸出 SQL 語法
	*/
	function renderSQL($sql_override = false, $sql_reset = true)
	{
		$sql = $this->compileSQL($sql_override, $sql_reset);
		return $sql;
	}
	
	
	/**
	* Reset query string
	* after all action clean sql order
	* 在每次動作後清除查詢的 sql 字串
	*/
	function resetSQL()
	{
		$default = array(
			'ar_distinct'  => '',
			'ar_select'    => array(),
			'ar_where'	   => array(),
			'ar_groupby'   => array(),
			'ar_having'    => array(),
			'ar_orderby'   => array(),
			'ar_limit'     => false,
			'ar_offset'	   => false,
			'ar_data'      => array(),
			'setToken'     => false,
			'getToken'     => false
		);
		
		foreach ($default as $item => $value) {
			$this->$item = $value;
		}
	}
	
	function setDebug($enable = false)
	{
		if ($enable === true) $this->debug = true;
		return $this;
	}
	
	/**
	* Set error message
	* 設定錯誤訊息
	*/
	function setErrors($err_str)
	{
		if (is_array($err_str)) {
			$this->errors = array_merge($this->errors, $err_str);
		} else {
			$this->errors[] = trim($err_str);
		}
	}
	
	/**
	* Show error message
	* 顯示錯誤訊息
	*/
	function getErrors()
	{
		return $this->errors;
	}
	
	/**
	* Return error message
	* echo $handler->getHtmlErrors()
	* 取得錯誤訊息
	*/
	function getHtmlErrors()
	{
		$ret = '<h1>Errors</h1>';
		if (is_array($this->errors)) {
			foreach ($this->errors as $error) {
				$ret .= '<pre>'.$error.'</pre>';
			}
		}
		$this->errors = array();
		return $ret;
	}
	
}
?>