<?php
/*
 You may not change or alter any portion of this comment or credits
 of supporting developers from this source code or any supporting source code
 which is considered copyrighted (c) material of the original comment or credit authors.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
*/

xoops_load('gui', 'system');

/**
 * Xoops GUI Classic Class
 * 
 * @copyright   The XOOPS project http://sf.net/projects/xoops/
 * @license     7-ELEVEN License - Always Open
 * @package     System
 * @usbpackage  GUI
 * @since       2.4.3
 * @author      RB Lin
 * @emaul       xtheme@gmail.com
 * @web         http://xoops.tnc.edu.tw/
 * @url         http://code.google.com/p/xoops-rb-framework/
 * @version     0.33
 * @update      2010-04-09
 */

class XoopsGuiClassic extends /* implements */ XoopsSystemGui
{
	protected $option = array();
	
	function __set($k, $c) {
		$this->option[$k]= $c;
	}
	
	function __get($k) {
		return $this->option[$k];
	}
	
	function __construct() {
		
		global $xoopsModule;
		
		include_once dirname(__FILE__).'/include/functions.php';
		$is_index = ( !is_object($xoopsModule) && !isset($_GET['xoopsorgnews']) ) ? true : false;
		
		$folderName = strtolower(str_replace('XoopsGui', '', get_class()));
		
		$option = array(
			'folderName'     => $folderName,
			'theme_path'     => XOOPS_ROOT_PATH.'/modules/system/class/gui/'.$folderName,
			'theme_url'      => XOOPS_URL.'/modules/system/class/gui/'.$folderName,
			'is_index' => $is_index,
			'framework_url'  => XOOPS_URL.'/Frameworks',
			'called_modules' => array(),
			'bind_module' => array(),
			'redirect_msg' => '',
			'redirect_time' => '',
			'gui_conf' => array(),
			'ero_msgs' => array()
		);
		
		foreach ($option as $k => $c ) {
			$this->option[$k] = $c;
		}
		
		// load gui configs
		$data = array();
		require_once dirname(__FILE__).'/include/data_db.php';
		foreach ($data as $k => $v ) {
			$this->option['gui_conf'][$k] = gui_decode($v);
		}
		
		// save redirect cookie info to smarty
		if(!empty($_COOKIE['redirect_message'])) {
			$this->redirect_msg = $_COOKIE['redirect_message'];
			$this->redirect_time = $_COOKIE['redirect_time'];
			setcookie('redirect_message', '', time()-1); 
			setcookie('redirect_time', '', time()-1); 
		}
		
		// show / hide panel (toolbar)
		if (!isset($_COOKIE['hide_panel'])) setcookie('hide_panel', 0, time() + 3600 * 24, '/');
    }

    function XoopsGuiClassic() {
        $this->__construct();
    }

    // validate theme can use or not
	function validate() {
		$ret = ( version_compare(PHP_VERSION, '5.2.0', '<') ) ? false : true; // check PHP5 && JSON
		
		if ($ret)
		$ret = ( substr(XOOPS_VERSION, 6, 3) <> 2.4 ) ? false : true;
		
		return $ret;
    }
	
	function header() {
		parent::header();
		global $xoopsUser, $xoopsModule, $xoopsLogger;
		
		$xoopsLogger->startTime('ACP init');
		
		$this->template->assign(array(
			'gui' => $this->gui_conf, // GUI config
			'welcome' => sprintf(_AD_WELCOME, $xoopsUser->getVar('uname')),
			'is_index' => $this->is_index,
			'prem_menu' => $this->prefMenu(),
			'footer_menu' => $this->systemMenu()
		));
		if (true === $this->is_index):
		$this->template->assign(array(
			'xoops_pagetitle' => _AD_ACP_HOME,
			'sysinfo' => $this->systemInfo(),
			'modules' => $this->modulesInfo()
		));
		endif;

		$xoopsLogger->startTime('Rewrite init');
		$this->headerRewrite();
		$xoopsLogger->stopTime('Rewrite init');
		
		/*
		 * ### navigation menu start ###
		 */
		$module_handler =& xoops_gethandler('module');
		$module_perm_handler =& xoops_gethandler('groupperm');
		$nav_menu = array();
		
		/*
		 * get my custom navigation links set  'include/custom_menu.php'
		 */
		include dirname(__FILE__).'/include/custom_menu.php';
		foreach ($custom as $key => $type) {
			$nav_menu[$key]['title'] = $type['title'];
			$nav_menu[$key]['link'] = $type['link'];
			foreach ($type['list'] as $kk => $dir) {
				$mod_obj = $module_handler->getByDirname($dir);
				if (!$mod_obj || !$mod_obj->getVar('isactive')) continue;
				
				$is_admin = $module_perm_handler->checkRight('module_admin', $mod_obj->getVar('mid'), $xoopsUser->getGroups());
				if (!$is_admin) continue;
				
				$nav_menu[$key]['list'][$kk] = $this->generateNav($mod_obj);
			}
		}
		
		/*
		 * get all install and active modules, but preclude custom items 'include/custom_menu.php'
		 */
		++$key;
		$nav_menu[$key]['title'] = _AD_MODULES;
		$nav_menu[$key]['link'] = _AD_LINK_NULL;
		$criteria = new CriteriaCompo();
		$criteria->add(new Criteria('hasadmin', 1));
		$criteria->add(new Criteria('isactive', 1));
		$criteria->setSort('mid');
		$module_obj = $module_handler->getObjects($criteria);
		foreach (array_keys($module_obj) as $m) {
			$mod_obj = $module_obj[$m];
			if (!$mod_obj || !$mod_obj->getVar('isactive')) continue;
			
			// preclude called modules
			if (in_array($module_obj[$m]->getVar('dirname', 'n'), $this->called_modules)) continue;
			
			$is_admin = $module_perm_handler->checkRight('module_admin', $mod_obj->getVar('mid'), $xoopsUser->getGroups());
			if (!$is_admin) continue;
			
			$nav_menu[$key]['list'][$m] = $this->generateNav($mod_obj);
		}
		
		/*
		 * get links 'include/custom_links.php'
		 */
		include dirname(__FILE__).'/include/custom_links.php';
		
		++$key;
		$nav_menu[$key]['title'] = _AD_XOOPS_LINKS;
		$nav_menu[$key]['link'] = _AD_LINK_NULL;
		foreach ($somelinks as $link) {
			$nav_menu[$key]['list'][] = array(
				'title' => $link['title'],
				'link'  => $link['link']
			);
		}
		
		++$key;
		$nav_menu[$key]['title'] = _AD_CREDIT;
		$nav_menu[$key]['link'] = _AD_LINK_NULL;
		foreach ($credit as $link) {
			$nav_menu[$key]['list'][] = array(
				'title' => $link['title'],
				'link'  => $link['link']
			);
		}
		/*
		 * ### navigation menu end ###
		 */
		
		/*
		 * assign to template
		 */
		$this->template->assign('nav_menu',    $nav_menu);
		$this->template->assign('module_menu', $this->bind_module);
		
		/*
		 * system (quick) configs
		 * enable some system configs for our customers without System Admin rights
		 * support conf_formtype : text / yesno / select
		 */
		$allow_config = array('debug_mode');
		$this->template->assign('sys_conf', $this->confMenu($allow_config));
		
		//echo '$_COOKIE[\'hide_panel\'] = '.$_COOKIE['hide_panel'];
		
		/*
		 * custom error message
		 */
		foreach ($this->ero_msgs as $chijo) {
			xoops_error($chijo);
			echo '<br />';
		}
    }
	
	function footer() {
		global $xoopsLogger;
		$xoopsLogger->activated = true;
		$this->template->assign('xoops_dumptime', sprintf(_AD_DUMP_TIME, number_format($xoopsLogger->dumpTime(), 5, '.', '')));
		$xoopsLogger->stopTime('ACP init');
		parent::footer();
	}
	
	// ######## protected function ########
	
	protected function setError($msg) {
		$msg = (is_array($msg)) ? $msg : (array)$msg;
		$this->ero_msgs = array_merge($this->ero_msgs, $msg);
	}
	
	protected function headerRewrite() {
		$extra_js  = "var \$guiurl = '".$this->theme_url."';\n";
		$extra_js .= "var \$lang_process = '"._AD_PROCESS."';\n";
		$extra_js .= "var \$feed_url = '"._AD_FEED_URL."';\n";
		$extra_js .= "$(document).ready(function(){\n";
		$extra_js .= "$.getScript(\$guiurl + '/js/gui.main.js');\n";
		$extra_js .= "$.getScript(\$guiurl + '/js/gui.feed.js');\n";
		if ($this->is_index):
		$extra_js .= "$.getScript(\$guiurl + '/js/gui.config.js');\n";
		endif;
		if ($this->redirect_msg != ''):
		$extra_js .= "showMessage('".$this->redirect_msg."', ".$this->redirect_time.");\n";
		endif;
		$extra_js .= "});\n";
		// use jGrowl show redirect message
		$extra_js .= "function showMessage(str, msec) {\n";
		$extra_js .= "if (!parseInt(msec)) msec = 2000;\n";
		$extra_js .= "if (str != '') $.jGrowl(str, { life:msec, position:'center', speed:'slow' });\n";
		$extra_js .= "}";
		
		// header rewrite handler, default use RBF class.load.php
		$hander_path = XOOPS_ROOT_PATH.'/Frameworks/rb/class.load.php';
		$hander_exists = ( !file_exists($hander_path) ) ? false : true;
		if (!$hander_exists):
			global $xoTheme; // Project A require 'Jackie Chan's nose'
			$xoTheme->addStylesheet($this->framework_url.'/rb/style.css');
			$xoTheme->addScript('http://ajax.googleapis.com/ajax/libs/jquery/1.4.2/jquery.min.js');
			$xoTheme->addScript('http://ajax.googleapis.com/ajax/libs/jqueryui/1.7.2/jquery-ui.min.js');
			$xoTheme->addStylesheet($this->framework_url.'/rb/style.css');
			$xoTheme->addScript(XOOPS_URL.'/include/xoops.js');
			$xoTheme->addScript($this->framework_url.'/jquery/jquery.livequery.js');
			$xoTheme->addScript($this->framework_url.'/jquery/jquery.jfeed.pack.js');
			//$xoTheme->addScript($this->framework_url.'/jquery/jquery.960grid.js');
			$xoTheme->addScript($this->framework_url.'/jquery/jgrowl/jquery.jgrowl.js');
			$xoTheme->addStylesheet($this->framework_url.'/jquery/jgrowl/jquery.jgrowl.css');
			$xoTheme->addScript($this->theme_url.'/js/gui.panel.js');
			$xoTheme->addScript('', null, $extra_js);
			$xoTheme->addStylesheet($this->theme_url.'/css/style.css');
			//$ero_msg = sprintf(_AD_WARNING_RBF, $hander_path);
			//$this->setError($ero_msg);
		else:
			include_once $hander_path; // Project B
			$loader = new RbLoad();
			$loader->loadJs(XOOPS_URL.'/include/xoops.js')
			->loadCss('rb/style.css')
			->loadJs('jquery/jquery.livequery.js')
			->loadJs('jquery/jquery.jfeed.pack.js')
			//->loadJs('jquery/jquery.960grid.js')
			->loadPlugin('jquery/jgrowl')
			->setBaseUrl($this->theme_url)
			->loadCss('css/style.css')
			->loadJs('js/gui.panel.js')
			->writeJs($extra_js)->rewrite();
		endif;
    }
	
	protected function generateNav($mod_obj) {
		global $xoopsModule;
		
		$mod_dir = $mod_obj->getVar('dirname', 'n');
		$mod_mid = $mod_obj->getVar('mid');
		$this->called_modules = array_merge($this->called_modules, (array)$mod_dir);
		
		$mod_info = $mod_obj->getInfo();
		if (!empty($mod_info['adminindex'])) {
			$mod_link = XOOPS_URL.'/modules/'.$mod_dir.'/'.$mod_info['adminindex'];
		} else {
			$mod_link = XOOPS_URL.'/modules/system/admin.php?fct=preferences&amp;op=showmod&amp;mod='.$mod_mid;
		}
		
		$arr = array(
			'title' => $mod_obj->name(),
			'link'  => $mod_link,
			'icon'  => '', // $mod_info['adminicon']
			'submenu' => false
		);
		// has admin menu
		if (!empty($mod_info['adminmenu'])) {
			$mod_submenu = $mod_obj->getAdminMenu();
			if(isset($mod_submenu) && is_array($mod_submenu)) {
				foreach ($mod_submenu as $submenu) {
					$arr['submenu'][] = array(
						'title' => $submenu['title'],
						'link' => XOOPS_URL.'/modules/'.$mod_dir.'/'.$submenu['link'],
						'icon' => ''
					);
				}
			}
		}
		// has config
		if ($mod_obj->getVar('hasconfig')) {
			$arr['submenu'][] = array(
				'title' => _AD_MOD_SETTING,
				'link' => XOOPS_URL.'/modules/system/admin.php?fct=preferences&amp;op=showmod&amp;mod='.$mod_mid,
				'icon' => $this->theme_url.'/images/cog.png'
			);
		}
		// has front
		if ($mod_obj->getVar('hasmain')) {
			$arr['submenu'][] = array(
				'title' => _AD_MOD_FRONT,
				'link' => XOOPS_URL.'/modules/'.$mod_dir.'/',
				'icon' => $this->theme_url.'/images/layout.png'
			);
		}
		// bind now module menu
		if (is_object($xoopsModule) && ($xoopsModule->getVar('dirname') == $mod_dir)) {
			$this->bind_module = $arr['submenu'];
		}
		unset($mod_obj);
		return $arr;
	}
	
	protected function statusIcon($status) {
		if ($status) {
			$str = '<img src="'.$this->theme_url.'/images/action_check.gif" alt="ON" />';
		} else {
			$str = '<img src="'.$this->theme_url.'/images/action_delete.gif" alt="OFF" />';
		}
		return $str;
	}
	
	protected function isSec($val) {
		$str = $val._AD_SECOND;
		return $str;
	}
	
	protected function iniFormat($conf, $callFunc = '') {
		$value = ini_get($conf);
		if ($callFunc != '') {
			$value = $this->$callFunc($value);
		}
		return $value;
	}
	
	protected function systemInfo() {
		$i = 0;
		$sysinfo[$i]['label'] = sprintf(_MD_CPANEL_VERSION, 'XOOPS');
		$sysinfo[$i]['value'] = XOOPS_VERSION;
		$i++;
		$sysinfo[$i]['label'] = sprintf(_MD_CPANEL_VERSION, 'PHP');
		$sysinfo[$i]['value'] = PHP_VERSION;
		$i++;
		$sysinfo[$i]['label'] = sprintf(_MD_CPANEL_VERSION, 'MySQL');
		$sysinfo[$i]['value'] = mysql_get_server_info();
		$i++;
		$sysinfo[$i]['label'] = sprintf(_MD_CPANEL_VERSION, 'Server API');
		$sysinfo[$i]['value'] = PHP_SAPI;
		$i++;
		$sysinfo[$i]['label'] = sprintf(_MD_CPANEL_VERSION, 'OS');
		$sysinfo[$i]['value'] = PHP_OS;
		/*
		 * custom check functions here, use 'display title' => 'function name'
		 */
		$checkFunctions = array(
			'GD library' => 'gd_info',
			'iconv',
			'mbstring' => 'mb_strlen',
			'cURL' => 'curl_init'
		);
		foreach ($checkFunctions as $lab => $func) {
			$i++;
			$sysinfo[$i]['label'] = (!empty($lab)) ? $lab : $func;
			$sysinfo[$i]['value'] = $this->statusIcon(function_exists($func));
		}
		/*
		 * custom check configs here, use 'config name' => 'custom format mrthod'
		 */
		$checkConfiguration = array(
			'safe_mode' => 'statusIcon',
			'register_globals' => 'statusIcon',
			'magic_quotes_gpc' => 'statusIcon',
			'allow_url_fopen' => 'statusIcon',
			'allow_url_include' => 'statusIcon',
			'memory_limit' => '',
			'max_execution_time' => 'isSec',
			'max_input_time' => 'isSec',
			'post_max_size' => '',
			'upload_max_filesize' => ''
		);
		foreach ($checkConfiguration as $conf => $format) {
			$i++;
			$sysinfo[$i]['label'] = $conf;
			$sysinfo[$i]['value'] = $this->iniFormat($conf, $format);
		}
		
		return $sysinfo;
	}
	
	protected function modulesInfo() {
		global $xoopsUser;
		// Install Modules Info
		$install = array();
		$module_handler =& xoops_gethandler('module');
		$criteria = new CriteriaCompo();
		$criteria->add(new Criteria('hasadmin', 1));
		$criteria->add(new Criteria('isactive', 1));
		$criteria->setSort('mid');
		$module_obj = $module_handler->getObjects($criteria);
		$module_perm_handler =& xoops_gethandler('groupperm');
		foreach (array_keys($module_obj) as $m) {
			$module = array();
			$is_admin = $module_perm_handler->checkRight('module_admin', $module_obj[$m]->getVar('mid'), $xoopsUser->getGroups());
			if ($is_admin) {
				$module['title'] = $module_obj[$m]->name();
				$module['dir'] = $module_obj[$m]->getVar('dirname', 'n');
				$module['mid'] = $module_obj[$m]->getVar('mid');
				$info = $module_obj[$m]->getInfo();
				if ($info['image']) {
					$icon = XOOPS_URL.'/modules/'.$module['dir'].'/'.$info['image'];
				} else {
					$icon = $this->theme_url.'/images/slogo.gif';
				}
				$module['icon'] = $icon;
				if ($module_obj[$m]->getVar('hasmain')) {
					$module['front_link'] = XOOPS_URL.'/modules/'.$module['dir'].'/';
				}
				if ($module_obj[$m]->getVar('hasadmin')) {
					if (!empty($info['adminindex'])) {
						$module['backend_link'] = XOOPS_URL.'/modules/'.$module['dir'].'/'.$info['adminindex'];
					} else {
						$module['backend_link'] = XOOPS_URL.'/modules/system/admin.php?fct=preferences&amp;op=showmod&amp;mod='.$module['mid'];
					}
				}
				$install[] = $module;
				unset($info);
			}
		}
		return $install;
    }
	
	protected function systemMenu() {
		include dirname(__FILE__).'/include/system_menu.php';
		return $adminmenu;
	}
	
	protected function prefMenu() {
		global $xoopsConfig;
		$fct = 'preferences';
		if (!@include_once XOOPS_ROOT_PATH.'/modules/system/language/'.$xoopsConfig['language'].'/admin/'.$fct.'.php') {
			include_once XOOPS_ROOT_PATH.'/modules/system/language/english/admin/'.$fct.'.php';
		}
		
		$confcat_handler = xoops_gethandler('configcategory');
        $confcats = $confcat_handler->getObjects();
        $catcount = count($confcats);
        for ($i = 0; $i < $catcount; $i++) {
			$pref[$i]['title'] = constant($confcats[$i]->getVar('confcat_name'));
			$pref[$i]['link'] = XOOPS_URL.'/modules/system/admin.php?fct=preferences&op=show&confcat_id='.$confcats[$i]->getVar('confcat_id');
		}
		
		return $pref;
	}
	
	protected function confMenu($enable_config = array('banners')) {
		global $xoopsConfig;
		$fct = 'preferences';
		if (!@include_once XOOPS_ROOT_PATH.'/modules/system/language/'.$xoopsConfig['language'].'/admin/'.$fct.'.php') {
			include_once XOOPS_ROOT_PATH.'/modules/system/language/english/admin/'.$fct.'.php';
		}
		
		$arr = array();
		$config_handler =& xoops_gethandler('config');
		
		$criteria = new CriteriaCompo();
		$allow_config = array('sitename', 'debug_mode');
		foreach($enable_config as $conf_name) {
			$criteria->add(new Criteria('conf_name', $conf_name), 'OR');
		}
		$criteria->setSort('conf_order');
		
		$configs = $config_handler->getConfigs($criteria);
		foreach (array_keys($configs) as $i) {
			$conf = array();
			$conf['id']    = $configs[$i]->getVar('conf_id');
			$conf['title'] = constant($configs[$i]->getVar('conf_title'));
			$conf['name']  = $configs[$i]->getVar('conf_name');
			$conf['value'] = $configs[$i]->getVar('conf_value');
			$conf['formtype'] = $configs[$i]->getVar('conf_formtype');
			$conf['valuetype'] = $configs[$i]->getVar('conf_valuetype');
			
			$options = $config_handler->getConfigOptions(new Criteria('conf_id', $configs[$i]->getVar('conf_id')));
			foreach($options as $opt) {
				$conf['options'][] = array(
					'val' => defined($opt->getVar('confop_value')) ? constant($opt->getVar('confop_value')) : $opt->getVar('confop_value'),
					'key' => defined($opt->getVar('confop_name'))  ? constant($opt->getVar('confop_name'))  : $opt->getVar('confop_name')
				);
			}
			$arr[] = $conf;
		}
		return $arr;
	}
	
}
?>